@@ -80,7 +80,7 @@ lxml_get_lisp_addr(xmlNodePtr xml_address)
     }else if (strcmp(str_afi,"lcaf") == 0){
 
     }else{
-        OOR_LOG(LDBG_2,"OOR_API->lxml_get_lisp_addr: Afi not suppoted: %s",str_afi);
+        OOR_LOG(LDBG_2,"OOR_API->lxml_get_lisp_addr: Afi not supported: %s",str_afi);
         return NULL;
     }
 
@@ -121,7 +121,7 @@ lxml_get_char_lisp_addr(xmlNodePtr xml_address, char *name, shash_t *lcaf_ht)
         shash_insert(lcaf_ht,strdup(name),laddr);
         return (strdup(name));
     }else{
-        OOR_LOG(LDBG_2,"OOR_API->lxml_get_char_lisp_addr: Afi not suppoted: %s",str_afi);
+        OOR_LOG(LDBG_2,"OOR_API->lxml_get_char_lisp_addr: Afi not supported: %s",str_afi);
         free(str_afi);
         return (NULL);
     }
@@ -205,7 +205,7 @@ lxml_lcaf_get_lisp_addr (xmlNodePtr xml_lcaf)
             xml_elp_node = lxml_get_next_node(xml_elp_node);
         }
     }else {
-        OOR_LOG(LDBG_2,"OOR_API->lxml_lcaf_get_lisp_addr: LCAF type not suppoted: %s",lcaf_type);
+        OOR_LOG(LDBG_2,"OOR_API->lxml_lcaf_get_lisp_addr: LCAF type not supported: %s",lcaf_type);
         return (NULL);
     }
     free(lcaf_type);
@@ -354,7 +354,7 @@ lxml_update_map_server_list(xmlNodePtr xml_map_servers, uint8_t proxy_reply,
         }
         free(key_type_aux);
         if (key_type != HMAC_SHA_1_96){
-            OOR_LOG(LERR, "Configuraton file: Only SHA-1 (1) authentication is supported");
+            OOR_LOG(LERR, "Configuration file: Only SHA-1 (1) authentication is supported");
             free(str_addr);
             free(key);
             return (BAD);
@@ -665,7 +665,7 @@ oor_api_xtr_mapdb_create(oor_api_connection_t *conn, oor_api_msg_hdr_t *hdr,
 {
     lisp_xtr_t *xtr;
     mapping_t *processed_mapping;
-    map_local_entry_t *map_loc_e;
+    map_local_entry_t *map_loc_e, *db_map_loc_e;
     shash_t *lcaf_ht;
     void *it;
     xmlDocPtr doc;
@@ -674,7 +674,10 @@ oor_api_xtr_mapdb_create(oor_api_connection_t *conn, oor_api_msg_hdr_t *hdr,
     xmlNodePtr xml_local_eid;
     conf_mapping_t *conf_mapping;
     glist_t *conf_mapping_list;
+    glist_t *smr_lcl_map_e_list;
+    glist_t *lcl_map_e_list;
     glist_entry_t *conf_map_it;
+    glist_entry_t *local_map_entry_it;
     int result_msg_len;
     uint8_t *result_msg;
     int ipv4_mapings = 0;
@@ -684,6 +687,8 @@ oor_api_xtr_mapdb_create(oor_api_connection_t *conn, oor_api_msg_hdr_t *hdr,
     OOR_LOG(LDBG_1, "OOR_API: Creating new local data base");
     lcaf_ht = shash_new_managed((free_value_fn_t)lisp_addr_del);
     conf_mapping_list = glist_new_managed((glist_del_fct)conf_mapping_destroy);
+    smr_lcl_map_e_list = glist_new();
+    lcl_map_e_list = glist_new();
 
     xtr = CONTAINER_OF(ctrl_dev, lisp_xtr_t, super);
 
@@ -703,18 +708,6 @@ oor_api_xtr_mapdb_create(oor_api_connection_t *conn, oor_api_msg_hdr_t *hdr,
     xmlFreeDoc(doc);
     doc = NULL;
 
-    /*
-     * Empty previous local database
-     */
-    /* Remove routing configuration for the eids */
-    local_map_db_foreach_entry(xtr->local_mdb, it) {
-        map_loc_e = (map_local_entry_t *)it;
-        ctrl_unregister_eid_prefix(ctrl_dev, map_local_entry_eid(map_loc_e));
-    } local_map_db_foreach_end;
-
-    /* Empty local database */
-    local_map_db_del(xtr->local_mdb);
-    xtr->local_mdb = local_map_db_new();
 
     /* We leverage on the OOR configuration subsystem to introduce
      * and process the configuration mappings into the system */
@@ -729,7 +722,7 @@ oor_api_xtr_mapdb_create(oor_api_connection_t *conn, oor_api_msg_hdr_t *hdr,
             goto err;
         }
         /* If dev is a mobile node, we can only have one IPv4 and one IPv6 mapping */
-        if (lisp_ctrl_dev_mode(ctrl_dev) == MN_MODE){
+        if (ctrl_dev_mode(ctrl_dev) == MN_MODE){
             eid_ip_afi = lisp_addr_ip_afi((lisp_addr_get_ip_pref_addr(mapping_eid(processed_mapping))));;
             if (eid_ip_afi == AF_INET){
                 ipv4_mapings ++;
@@ -750,25 +743,52 @@ oor_api_xtr_mapdb_create(oor_api_connection_t *conn, oor_api_msg_hdr_t *hdr,
             goto err;
         }
         if (xtr->fwd_policy->init_map_loc_policy_inf(
-                xtr->fwd_policy_dev_parm, map_loc_e, NULL,
-                xtr->fwd_policy->del_map_loc_policy_inf) != GOOD){
+                xtr->fwd_policy_dev_parm, map_loc_e, NULL) != GOOD){
             OOR_LOG(LDBG_2, "OOR_API: Couldn't initiate forward information for mapping with EID: %s",conf_mapping->eid_prefix);
             goto err;
         }
 
+        /* Classify entries that are already configured in the data base */
+        if ((db_map_loc_e = local_map_db_lookup_eid_exact(xtr->local_mdb, mapping_eid(processed_mapping))) != NULL){
+            /* If the mapping already exist but has been modified add to the list of mappings to SMR */
+            if (mapping_cmp(map_local_entry_mapping(db_map_loc_e), processed_mapping) != 0){
+                glist_add(map_loc_e,smr_lcl_map_e_list);
+            }
+        }
+        glist_add(map_loc_e,lcl_map_e_list);
+    }
+
+    /*
+     * Empty previous local database
+     */
+    /* Remove routing configuration for the eids */
+    local_map_db_foreach_entry(xtr->local_mdb, it) {
+        map_loc_e = (map_local_entry_t *)it;
+        ctrl_unregister_mapping_dp(ctrl_dev, map_local_entry_mapping(map_loc_e));
+    } local_map_db_foreach_end;
+
+    /* Empty local database */
+    local_map_db_del(xtr->local_mdb);
+    xtr->local_mdb = local_map_db_new();
+
+
+    /* Add the local map entries to the data base */
+    glist_for_each_entry(local_map_entry_it, lcl_map_e_list){
+        map_loc_e = (map_local_entry_t *)glist_entry_data(local_map_entry_it);
+
         if (add_local_db_map_local_entry(map_loc_e,xtr) != GOOD){
             OOR_LOG(LDBG_2, "OOR_API: Couldn't add mapping %s to local database",
-                    lisp_addr_to_char(&(processed_mapping->eid_prefix)));
+                    lisp_addr_to_char(map_local_entry_eid(map_loc_e)));
             goto err;
         }
 
         OOR_LOG(LDBG_1, "OOR_API: Updating data-plane for EID prefix %s",
-                lisp_addr_to_char(&(processed_mapping->eid_prefix)));
+                lisp_addr_to_char(map_local_entry_eid(map_loc_e)));
 
         /* Update the routing rules for the new EID */
-        if (ctrl_register_eid_prefix(ctrl_dev,mapping_eid(processed_mapping))!=GOOD){
+        if (ctrl_register_mapping_dp(ctrl_dev,map_local_entry_mapping(map_loc_e))!=GOOD){
             OOR_LOG(LERR, "OOR_API: Unable to update data-plane for mapping %s",
-                    lisp_addr_to_char(&(processed_mapping->eid_prefix)));
+                    lisp_addr_to_char(map_local_entry_eid(map_loc_e)));
             goto err;
         }
     }
@@ -776,8 +796,6 @@ oor_api_xtr_mapdb_create(oor_api_connection_t *conn, oor_api_msg_hdr_t *hdr,
     /* Update control with new added interfaces */
     ctrl_update_iface_info(ctrl_dev->ctrl);
 
-    glist_destroy(conf_mapping_list);
-    shash_destroy(lcaf_ht);
 
     OOR_LOG(LDBG_1, "OOR_API: New local data base created");
     OOR_LOG(LDBG_2, "************* %20s ***************", "Local EID Database");
@@ -787,15 +805,27 @@ oor_api_xtr_mapdb_create(oor_api_connection_t *conn, oor_api_msg_hdr_t *hdr,
     /* Reprogram Map Register for local EIDs */
     program_map_register(xtr);
 
+    /* SMR of the modified entries */
+    glist_for_each_entry(local_map_entry_it, smr_lcl_map_e_list){
+        map_loc_e = (map_local_entry_t *)glist_entry_data(local_map_entry_it);
+        send_smr_and_mreg_for_locl_mapping(xtr, map_loc_e);
+    }
 
     result_msg_len = oor_api_result_msg_new(&result_msg,hdr->device,hdr->target,hdr->operation,OOR_API_RES_OK);
     oor_api_send(conn,result_msg,result_msg_len,OOR_API_NOFLAGS);
 
+    glist_destroy(conf_mapping_list);
+    glist_destroy(smr_lcl_map_e_list);
+    glist_destroy(lcl_map_e_list);
+    shash_destroy(lcaf_ht);
+
     return (GOOD);
 
     err:
     //XXX if error, destroy mappings added to local mapdb? deattach locators from ifaces?
     glist_destroy(conf_mapping_list);
+    glist_destroy(smr_lcl_map_e_list);
+    glist_destroy(lcl_map_e_list);
     shash_destroy(lcaf_ht);
 
     if (doc != NULL){
@@ -827,7 +857,7 @@ oor_api_xtr_mapdb_delete(oor_api_connection_t *conn, oor_api_msg_hdr_t *hdr,
     /* Remove routing configuration for the eids */
     local_map_db_foreach_entry(xtr->local_mdb, it) {
         map_loc_e = (map_local_entry_t *)it;
-        ctrl_unregister_eid_prefix(ctrl_dev, map_local_entry_eid(map_loc_e));
+        ctrl_unregister_mapping_dp(ctrl_dev, map_local_entry_mapping(map_loc_e));
     } local_map_db_foreach_end;
 
     /* Empty local database */
@@ -903,17 +933,25 @@ oor_api_xtr_petrs_create(oor_api_connection_t *conn, oor_api_msg_hdr_t *hdr,
     doc = NULL;
 
     //Everything fine. We replace the old list with the new one
-    glist_remove_all(mapping_locators_lists(mcache_entry_mapping(xtr->petrs)));
+    glist_remove_all(mapping_locators_lists(mcache_entry_mapping(xtr->petrs_ipv4)));
+    glist_remove_all(mapping_locators_lists(mcache_entry_mapping(xtr->petrs_ipv6)));
     glist_for_each_entry(addr_it,str_addr_list){
         str_addr = (char *)glist_entry_data(addr_it);
-        add_proxy_etr_entry(xtr->petrs,str_addr,1,100);
+        add_proxy_etr_entry(xtr->petrs_ipv4,str_addr,1,100);
+        add_proxy_etr_entry(xtr->petrs_ipv6,str_addr,1,100);
     }
 
-    xtr->fwd_policy->updated_map_cache_inf(xtr->fwd_policy_dev_parm,xtr->petrs);
+
+    xtr->fwd_policy->updated_map_cache_inf(xtr->fwd_policy_dev_parm,xtr->petrs_ipv4);
+    notify_datap_rm_fwd_from_entry(&(xtr->super),mcache_entry_eid(xtr->petrs_ipv4),FALSE);
+
+    xtr->fwd_policy->updated_map_cache_inf(xtr->fwd_policy_dev_parm,xtr->petrs_ipv6);
+    notify_datap_rm_fwd_from_entry(&(xtr->super),mcache_entry_eid(xtr->petrs_ipv6),FALSE);
 
     OOR_LOG(LDBG_1, "OOR_API: List of Proxy ETRs successfully created");
     OOR_LOG(LDBG_1, "************************* Proxy ETRs List ****************************");
-    mapping_to_char(mcache_entry_mapping(xtr->petrs));
+    mapping_to_char(mcache_entry_mapping(xtr->petrs_ipv4));
+    mapping_to_char(mcache_entry_mapping(xtr->petrs_ipv6));
 
     result_msg_len = oor_api_result_msg_new(&result_msg,hdr->device,hdr->target,hdr->operation,OOR_API_RES_OK);
     oor_api_send(conn,result_msg,result_msg_len,OOR_API_NOFLAGS);
@@ -945,7 +983,8 @@ oor_api_xtr_petrs_delete(oor_api_connection_t *conn, oor_api_msg_hdr_t *hdr,
 
     xtr = CONTAINER_OF(ctrl_dev, lisp_xtr_t, super);
 
-    glist_remove_all(mapping_locators_lists(mcache_entry_mapping(xtr->petrs)));
+    glist_remove_all(mapping_locators_lists(mcache_entry_mapping(xtr->petrs_ipv4)));
+    glist_remove_all(mapping_locators_lists(mcache_entry_mapping(xtr->petrs_ipv6)));
 
     result_msg_len = oor_api_result_msg_new(&result_msg,hdr->device,hdr->target,hdr->operation,OOR_API_RES_OK);
     oor_api_send(conn,result_msg,result_msg_len,OOR_API_NOFLAGS);
@@ -968,7 +1007,7 @@ int
 
     switch (device){
     case OOR_API_DEV_XTR:
-        if (lisp_ctrl_dev_mode(ctrl_dev) != xTR_MODE && lisp_ctrl_dev_mode(ctrl_dev) != MN_MODE){
+        if (ctrl_dev_mode(ctrl_dev) != xTR_MODE && ctrl_dev_mode(ctrl_dev) != MN_MODE){
             OOR_LOG(LDBG_1, "OOR_API call = Call API from wrong device");
             break;
         }
@@ -1039,7 +1078,7 @@ int
         }
         break;
     case OOR_API_DEV_RTR:
-        if (lisp_ctrl_dev_mode(ctrl_dev) != RTR_MODE){
+        if (ctrl_dev_mode(ctrl_dev) != RTR_MODE){
             OOR_LOG(LDBG_1, "OOR_API call = Call API from wrong device");
             break;
         }
