@@ -44,21 +44,37 @@ parse_elp_list(cfg_t *cfg, shash_t *ht)
     lisp_addr_t *laddr;
     char *name;
     int i, j;
+    uint8_t good_elp;
 
     for(i = 0; i < cfg_size(cfg, "explicit-locator-path"); i++) {
         cfg_t *selp = cfg_getnsec(cfg, "explicit-locator-path", i);
         name = cfg_getstr(selp, "elp-name");
+        if (name == NULL){
+            OOR_LOG(LWRN, "Configuration file: explicit-locator-path requires an elp-name. Discarding ELP");
+            continue;
+        }
+
+        if (cfg_size(selp, "elp-node") == 0){
+            OOR_LOG(LWRN, "Configuration file: explicit-locator-path needs at least one elp node. Discarding ELP");
+            continue;
+        }
 
         elp = elp_type_new();
 
+        good_elp = TRUE;
         for (j = 0; j < cfg_size(selp, "elp-node");j++) {
             cfg_t *senode = cfg_getnsec(selp, "elp-node", j);
+            if (cfg_getstr(senode, "address") == NULL){
+                good_elp = FALSE;
+                OOR_LOG(LWRN, "Configuration file: elp-node needs at least the address field. Discarding ELP");
+                break;
+            }
             enode = xzalloc(sizeof(elp_node_t));
             enode->addr = lisp_addr_new();
             if (lisp_addr_ip_from_char(cfg_getstr(senode, "address"),
                     enode->addr) != GOOD) {
                 elp_node_del(enode);
-                OOR_LOG(LDBG_1, "parse_elp_list: Couldn't parse ELP node %s",
+                OOR_LOG(LWRN, "Configuration file: Couldn't parse ELP node %s",
                         cfg_getstr(senode, "address"));
                 continue;
             }
@@ -69,6 +85,11 @@ parse_elp_list(cfg_t *cfg, shash_t *ht)
             glist_add_tail(enode, elp->nodes);
         }
 
+        if (good_elp == FALSE){
+            elp_type_del(elp);
+            continue;
+        }
+
         laddr = lisp_addr_new_lafi(LM_AFI_LCAF);
         lisp_addr_lcaf_set_type(laddr, LCAF_EXPL_LOC_PATH);
         lisp_addr_lcaf_set_addr(laddr, elp);
@@ -180,11 +201,20 @@ parse_mapping_cfg_params(cfg_t *map, conf_mapping_t *conf_mapping, uint8_t is_lo
     conf_loc_iface_t *conf_loc_iface;
     int afi;
 
+    if (cfg_getstr(map, "eid-prefix") == NULL){
+        return (BAD);
+    }
+
     strcpy(conf_mapping->eid_prefix,cfg_getstr(map, "eid-prefix"));
     conf_mapping->iid = cfg_getint(map, "iid");
 
     for (ctr = 0; ctr < cfg_size(map, "rloc-address"); ctr++){
         rl = cfg_getnsec(map, "rloc-address", ctr);
+
+        if (cfg_getstr(rl, "address") == NULL){
+            OOR_LOG(LWRN, "Configuration file: Mapping %s with no RLOC address selected",conf_mapping->eid_prefix);
+            return (BAD);
+        }
         conf_loc = conf_loc_new_init(
                 cfg_getstr(rl, "address"),
                 cfg_getint(rl, "priority"),
@@ -206,6 +236,10 @@ parse_mapping_cfg_params(cfg_t *map, conf_mapping_t *conf_mapping, uint8_t is_lo
                 OOR_LOG(LERR,"Configuration file: The conf_loc_iface->ip_version of the locator should be 4 (IPv4) or 6 (IPv6)");
                 return (BAD);
             }
+            if (cfg_getstr(rl, "interface") == NULL){
+                OOR_LOG(LWRN, "Configuration file: Mapping %s with no RLOC interface selected",conf_mapping->eid_prefix);
+                return (BAD);
+            }
             conf_loc_iface = conf_loc_iface_new_init(
                     cfg_getstr(rl, "interface"),
                     afi,
@@ -229,7 +263,9 @@ parse_mapping(cfg_t *map, oor_ctrl_dev_t *dev, shash_t * lcaf_ht,
 
     conf_mapping = conf_mapping_new();
 
-    parse_mapping_cfg_params(map, conf_mapping, is_local);
+    if (parse_mapping_cfg_params(map, conf_mapping, is_local) != GOOD){
+        return (NULL);
+    }
     mapping = process_mapping_config(dev, lcaf_ht, conf_mapping, is_local);
 
     conf_mapping_destroy(conf_mapping);
@@ -263,24 +299,51 @@ parse_proxy_etrs(cfg_t *cfg, lisp_xtr_t *xtr)
 {
     int n,i;
     /* PROXY-ETR CONFIG */
-    n = cfg_size(cfg, "proxy-etr");
+    n = cfg_size(cfg, "proxy-etr-ipv4");
     for(i = 0; i < n; i++) {
-        cfg_t *petr = cfg_getnsec(cfg, "proxy-etr", i);
-        if (add_proxy_etr_entry(xtr->petrs,
+        cfg_t *petr = cfg_getnsec(cfg, "proxy-etr-ipv4", i);
+        if(cfg_getstr(petr, "address") == NULL){
+            OOR_LOG(LERR,"Configuration file: proxy-etr-ipv4 needs at least the address field");
+            return (BAD);
+        }
+
+        if (add_proxy_etr_entry(xtr->petrs_ipv4,
                 cfg_getstr(petr, "address"),
                 cfg_getint(petr, "priority"),
                 cfg_getint(petr, "weight")) == GOOD) {
-            OOR_LOG(LDBG_1, "Added %s to proxy-etr list", cfg_getstr(petr, "address"));
+            OOR_LOG(LDBG_1, "Added %s to proxy-etr list for IPv4 EIDs", cfg_getstr(petr, "address"));
+        } else{
+            OOR_LOG(LERR, "Can't add proxy-etr %s", cfg_getstr(petr, "address"));
+        }
+    }
+
+    n = cfg_size(cfg, "proxy-etr-ipv6");
+    for(i = 0; i < n; i++) {
+        cfg_t *petr = cfg_getnsec(cfg, "proxy-etr-ipv6", i);
+        if(cfg_getstr(petr, "address") == NULL){
+            OOR_LOG(LERR,"Configuration file: proxy-etr-ipv6 needs at least the address field");
+            return (BAD);
+        }
+
+        if (add_proxy_etr_entry(xtr->petrs_ipv6,
+                cfg_getstr(petr, "address"),
+                cfg_getint(petr, "priority"),
+                cfg_getint(petr, "weight")) == GOOD) {
+            OOR_LOG(LDBG_1, "Added %s to proxy-etr list for IPv6 EIDs", cfg_getstr(petr, "address"));
         } else{
             OOR_LOG(LERR, "Can't add proxy-etr %s", cfg_getstr(petr, "address"));
         }
     }
 
     /* Calculate forwarding info for petrs */
-    if (xtr->fwd_policy->init_map_cache_policy_inf(xtr->fwd_policy_dev_parm,xtr->petrs,
-            xtr->fwd_policy->del_map_cache_policy_inf) != GOOD){
+    if (xtr->fwd_policy->init_map_cache_policy_inf(xtr->fwd_policy_dev_parm,xtr->petrs_ipv4) != GOOD){
         OOR_LOG(LDBG_1, "parse_proxy_etrs: Couldn't initiate routing info for PeTRs!.");
-        mcache_entry_del(xtr->petrs);
+        mcache_entry_del(xtr->petrs_ipv4);
+        return(BAD);
+    }
+    if (xtr->fwd_policy->init_map_cache_policy_inf(xtr->fwd_policy_dev_parm,xtr->petrs_ipv6) != GOOD){
+        OOR_LOG(LDBG_1, "parse_proxy_etrs: Couldn't initiate routing info for PeTRs!.");
+        mcache_entry_del(xtr->petrs_ipv6);
         return(BAD);
     }
     return (GOOD);
@@ -315,7 +378,7 @@ parse_database_mapping(cfg_t *cfg, lisp_xtr_t *xtr, shash_t *lcaf_ht)
     for (i = 0; i < n; i++) {
         mapping = parse_mapping(cfg_getnsec(cfg, "database-mapping", i),&(xtr->super),lcaf_ht,TRUE);
         if (mapping == NULL){
-            continue;
+            return (BAD);
         }
         map_loc_e = map_local_entry_new_init(mapping);
         if (map_loc_e == NULL){
@@ -323,8 +386,7 @@ parse_database_mapping(cfg_t *cfg, lisp_xtr_t *xtr, shash_t *lcaf_ht)
             continue;
         }
         if (xtr->fwd_policy->init_map_loc_policy_inf(
-                xtr->fwd_policy_dev_parm,map_loc_e,NULL,
-                xtr->fwd_policy->del_map_loc_policy_inf)!= GOOD){
+                xtr->fwd_policy_dev_parm,map_loc_e,NULL)!= GOOD){
             OOR_LOG(LERR, "Couldn't inititate forward information for mapping with EID: %s. Discarding it...",
                     lisp_addr_to_char(mapping_eid(mapping)));
             map_local_entry_del(map_loc_e);
@@ -349,7 +411,11 @@ configure_tunnel_router(cfg_t *cfg, lisp_xtr_t *xtr, shash_t *lcaf_ht)
     mapping_t *mapping;
 
     /* FWD POLICY STRUCTURES */
+#ifdef VPP
+    xtr->fwd_policy = fwd_policy_class_find("vpp_balancing");
+#else
     xtr->fwd_policy = fwd_policy_class_find("flow_balancing");
+#endif
     xtr->fwd_policy_dev_parm = xtr->fwd_policy->new_dev_policy_inf(ctrl_dev,NULL);
 
     if ((encap = cfg_getstr(cfg, "encapsulation")) != NULL) {
@@ -410,7 +476,7 @@ configure_tunnel_router(cfg_t *cfg, lisp_xtr_t *xtr, shash_t *lcaf_ht)
         if (mapping == NULL){
             OOR_LOG(LERR, "Can't add static Map Cache entry with EID prefix %s. Discarded ...",
                     cfg_getstr(smc, "eid-prefix"));
-            continue;
+            return(BAD);
         }
         if (mcache_lookup_exact(xtr->map_cache, mapping_eid(mapping)) == NULL){
             if (tr_mcache_add_static_mapping(xtr, mapping) == GOOD){
@@ -459,8 +525,16 @@ configure_rtr(cfg_t *cfg)
     if (n) {
         cfg_t *rifs = cfg_getsec(cfg, "rtr-ifaces");
         int nr = cfg_size(rifs, "rtr-iface");
+        if (nr == 0){
+            OOR_LOG(LERR, "Configuration file: RTR needs at least one data iface");
+        }
         for(i = 0; i < nr; i++) {
             cfg_t *ri = cfg_getnsec(rifs, "rtr-iface", i);
+            if (cfg_getstr(ri, "iface") == NULL){
+                OOR_LOG(LERR, "Configuration file: rtr-iface needs at least the iface name");
+                return (BAD);
+            }
+
             if (add_rtr_iface(xtr,
                     cfg_getstr(ri, "iface"),
                     cfg_getint(ri, "ip_version"),
@@ -480,7 +554,7 @@ configure_rtr(cfg_t *cfg)
     for (i = 0; i < n; i++) {
         mapping = parse_mapping(cfg_getnsec(cfg, "rtr-database-mapping",i),&(xtr->super),lcaf_ht,TRUE);
         if (mapping == NULL){
-            continue;
+            return (BAD);
         }
         map_loc_e = map_local_entry_new_init(mapping);
         if (map_loc_e == NULL){
@@ -489,8 +563,7 @@ configure_rtr(cfg_t *cfg)
         }
 
         if (xtr->fwd_policy->init_map_loc_policy_inf(
-                xtr->fwd_policy_dev_parm,map_loc_e,NULL,
-                xtr->fwd_policy->del_map_loc_policy_inf) != GOOD){
+                xtr->fwd_policy_dev_parm,map_loc_e,NULL) != GOOD){
             OOR_LOG(LERR, "Couldn't initiate forward information for rtr database mapping with EID: %s. Discarding it...",
                     lisp_addr_to_char(mapping_eid(mapping)));
             map_local_entry_del(map_loc_e);
@@ -535,7 +608,9 @@ configure_xtr(cfg_t *cfg)
 
     xtr->nat_aware = cfg_getbool(cfg, "nat_traversal_support") ? TRUE:FALSE;
     if(xtr->nat_aware){
-        nat_set_xTR_ID(xtr);
+        if (nat_set_xTR_ID(xtr) != GOOD){
+        	return (BAD);
+        }
         nat_set_site_ID(xtr, 0);
         default_rloc_afi = AF_INET;
         OOR_LOG(LDBG_1, "NAT support enabled. Set defaul RLOC to IPv4 family");
@@ -584,7 +659,10 @@ configure_mn(cfg_t *cfg)
 
     xtr->nat_aware = cfg_getbool(cfg, "nat_traversal_support") ? TRUE:FALSE;
     if(xtr->nat_aware){
-        nat_set_xTR_ID(xtr);
+        if (nat_set_xTR_ID(xtr) != GOOD){
+        	printf("aaaaaaaaaaaaa\n");
+        	return (BAD);
+        }
         nat_set_site_ID(xtr, 0);
         default_rloc_afi = AF_INET;
         OOR_LOG(LDBG_1, "NAT support enabled. Set defaul RLOC to IPv4 family");
@@ -617,7 +695,7 @@ int
 configure_ms(cfg_t *cfg)
 {
     char *iface_name;
-    iface_t *iface;
+    iface_t *iface=NULL;
     lisp_site_prefix_t *site;
     shash_t *lcaf_ht;
     int i;
@@ -641,25 +719,27 @@ configure_ms(cfg_t *cfg)
     if (iface_name) {
         iface = add_interface(iface_name);
         if (iface == NULL) {
+            OOR_LOG(LERR, "Configuration file: Couldn't add the control iface of the Map Server");
             return(BAD);
         }
+    }else{
+	/* we have no iface_name, so also iface is missing */
+        OOR_LOG(LERR, "Configuration file: Specify the control iface of the Map Server");
+        return(BAD);
     }
 
-    if (iface_address(iface, AF_INET) == NULL){
-        iface_setup_addr(iface, AF_INET);
-        data_plane->datap_add_iface_addr(iface,AF_INET);
-        lctrl->control_data_plane->control_dp_add_iface_addr(lctrl,iface,AF_INET);
-    }
-
-    if (iface_address(iface, AF_INET6) == NULL){
-        iface_setup_addr(iface, AF_INET6);
-        data_plane->datap_add_iface_addr(iface,AF_INET6);
-        lctrl->control_data_plane->control_dp_add_iface_addr(lctrl,iface,AF_INET6);
-    }
+    iface_configure (iface, AF_INET);
+    iface_configure (iface, AF_INET6);
 
     /* LISP-SITE CONFIG */
     for (i = 0; i < cfg_size(cfg, "lisp-site"); i++) {
         cfg_t *ls = cfg_getnsec(cfg, "lisp-site", i);
+
+        if (cfg_getstr(ls, "eid-prefix") == NULL || cfg_getstr(ls, "key") == NULL){
+            OOR_LOG(LERR, "Configuration file: MS LISP site requires at least an eid-prefix and a key");
+            return (BAD);
+        }
+
         site = build_lisp_site_prefix(ms,
                 cfg_getstr(ls, "eid-prefix"),
                 cfg_getint(ls, "iid"),
@@ -695,7 +775,7 @@ configure_ms(cfg_t *cfg)
         if (mapping == NULL){
             OOR_LOG(LERR, "Can't create static register site for %s",
                     cfg_getstr(mss, "eid-prefix"));
-            continue;
+            return (BAD);
         }
         /* If the mapping doesn't exist, add it the the database */
         if (mdb_lookup_entry_exact(ms->reg_sites_db, mapping_eid(mapping)) == NULL){
@@ -721,7 +801,7 @@ configure_ms(cfg_t *cfg)
 }
 
 int
-handle_config_file(char **oor_conf_file)
+handle_config_file()
 {
     int ret;
     cfg_t *cfg;
@@ -731,7 +811,7 @@ handle_config_file(char **oor_conf_file)
     /* xTR specific */
     static cfg_opt_t map_server_opts[] = {
             CFG_STR("address",              0, CFGF_NONE),
-            CFG_INT("key-type",             0, CFGF_NONE),
+            CFG_INT("key-type",             1, CFGF_NONE),
             CFG_STR("key",                  0, CFGF_NONE),
             CFG_BOOL("proxy-reply", cfg_false, CFGF_NONE),
             CFG_END()
@@ -739,16 +819,16 @@ handle_config_file(char **oor_conf_file)
 
     static cfg_opt_t rloc_address_opts[] = {
             CFG_STR("address",       0, CFGF_NONE),
-            CFG_INT("priority",      0, CFGF_NONE),
-            CFG_INT("weight",        0, CFGF_NONE),
+            CFG_INT("priority",      1, CFGF_NONE),
+            CFG_INT("weight",        100, CFGF_NONE),
             CFG_END()
     };
 
     static cfg_opt_t rloc_iface_opts[] = {
             CFG_STR("interface",     0, CFGF_NONE),
-            CFG_INT("ip_version",    0, CFGF_NONE),
-            CFG_INT("priority",      0, CFGF_NONE),
-            CFG_INT("weight",        0, CFGF_NONE),
+            CFG_INT("ip_version",    4, CFGF_NONE),
+            CFG_INT("priority",      1, CFGF_NONE),
+            CFG_INT("weight",        100, CFGF_NONE),
             CFG_END()
     };
 
@@ -769,16 +849,16 @@ handle_config_file(char **oor_conf_file)
 
     static cfg_opt_t petr_mapping_opts[] = {
             CFG_STR("address",              0, CFGF_NONE),
-            CFG_INT("priority",           255, CFGF_NONE),
-            CFG_INT("weight",               0, CFGF_NONE),
+            CFG_INT("priority",             1, CFGF_NONE),
+            CFG_INT("weight",               100, CFGF_NONE),
             CFG_END()
     };
 
     static cfg_opt_t rtr_iface_opts[] = {
             CFG_STR("iface",                0, CFGF_NONE),
-            CFG_INT("ip_version",           0, CFGF_NONE),
-            CFG_INT("priority",             255, CFGF_NONE),
-            CFG_INT("weight",               0, CFGF_NONE),
+            CFG_INT("ip_version",           4, CFGF_NONE),
+            CFG_INT("priority",             1, CFGF_NONE),
+            CFG_INT("weight",               100, CFGF_NONE),
             CFG_END()
     };
 
@@ -789,8 +869,8 @@ handle_config_file(char **oor_conf_file)
 
     static cfg_opt_t rloc_probing_opts[] = {
             CFG_INT("rloc-probe-interval",           0, CFGF_NONE),
-            CFG_INT("rloc-probe-retries",            0, CFGF_NONE),
-            CFG_INT("rloc-probe-retries-interval",   0, CFGF_NONE),
+            CFG_INT("rloc-probe-retries",            3, CFGF_NONE),
+            CFG_INT("rloc-probe-retries-interval",   10, CFGF_NONE),
             CFG_END()
     };
 
@@ -834,7 +914,7 @@ handle_config_file(char **oor_conf_file)
     static cfg_opt_t lisp_site_opts[] = {
             CFG_STR("eid-prefix",               0, CFGF_NONE),
             CFG_INT("iid",                      0, CFGF_NONE),
-            CFG_INT("key-type",                 0, CFGF_NONE),
+            CFG_INT("key-type",                 1, CFGF_NONE),
             CFG_STR("key",                      0, CFGF_NONE),
             CFG_BOOL("accept-more-specifics",   cfg_false, CFGF_NONE),
             CFG_BOOL("proxy-reply",             cfg_false, CFGF_NONE),
@@ -849,8 +929,9 @@ handle_config_file(char **oor_conf_file)
             CFG_SEC("static-map-cache",     map_cache_mapping_opts, CFGF_MULTI),
             CFG_SEC("map-server",           map_server_opts,        CFGF_MULTI),
             CFG_SEC("rtr-ifaces",           rtr_ifaces_opts,        CFGF_MULTI),
-            CFG_SEC("proxy-etr",            petr_mapping_opts,      CFGF_MULTI),
-            CFG_STR("encapsulation",        0,                      CFGF_NONE),
+            CFG_SEC("proxy-etr-ipv4",       petr_mapping_opts,      CFGF_MULTI),
+            CFG_SEC("proxy-etr-ipv6",       petr_mapping_opts,      CFGF_MULTI),
+            CFG_STR("encapsulation",        "LISP",                 CFGF_NONE),
             CFG_SEC("rloc-probing",         rloc_probing_opts,      CFGF_MULTI),
             CFG_INT("map-request-retries",  0, CFGF_NONE),
             CFG_INT("control-port",         0, CFGF_NONE),
@@ -875,8 +956,8 @@ handle_config_file(char **oor_conf_file)
             CFG_END()
     };
 
-    if (*oor_conf_file == NULL){
-        *oor_conf_file = strdup("/etc/oor.conf");
+    if (config_file == NULL){
+        config_file = strdup("/etc/oor.conf");
     }
 
     /*
@@ -884,11 +965,11 @@ handle_config_file(char **oor_conf_file)
      */
 
     cfg = cfg_init(opts, CFGF_NOCASE);
-    ret = cfg_parse(cfg, *oor_conf_file);
+    ret = cfg_parse(cfg, config_file);
 
 
     if (ret == CFG_FILE_ERROR) {
-        OOR_LOG(LCRIT, "Couldn't find config file %s, exiting...", config_file);
+        OOR_LOG(LCRIT, "Couldn't find config file %s. If you are useing OOR in daemon mode, please indicate a full path file.", config_file);
         cfg_free(cfg);
         return (BAD);
     } else if(ret == CFG_PARSE_ERROR) {
@@ -927,7 +1008,6 @@ handle_config_file(char **oor_conf_file)
     if (daemonize == TRUE){
         open_log_file(log_file);
     }
-
     mode = cfg_getstr(cfg, "operating-mode");
     if (mode) {
         if (strcmp(mode, "xTR") == 0) {
@@ -946,7 +1026,7 @@ handle_config_file(char **oor_conf_file)
     }
 
     cfg_free(cfg);
-    return(GOOD);
+    return(ret);
 }
 
 /*
