@@ -90,7 +90,7 @@ parse_elp_node(
 /********************************** FUNCTIONS ********************************/
 
 int
-handle_config_file(char **uci_conf_file_path)
+handle_config_file()
 {
     char *uci_conf_dir;
     char *uci_conf_file;
@@ -104,8 +104,8 @@ handle_config_file(char **uci_conf_file_path)
     int res = BAD;
 
 
-    if (*uci_conf_file_path == NULL){
-        *uci_conf_file_path = strdup("/etc/config/oor");
+    if (config_file == NULL){
+    	config_file = strdup("/etc/config/oor");
     }
 
     ctx = uci_alloc_context();
@@ -115,8 +115,8 @@ handle_config_file(char **uci_conf_file_path)
         return (BAD);
     }
 
-    uci_conf_dir = dirname(strdup(*uci_conf_file_path));
-    uci_conf_file = basename(strdup(*uci_conf_file_path));
+    uci_conf_dir = dirname(strdup(config_file));
+    uci_conf_file = basename(strdup(config_file));
 
 
     uci_set_confdir(ctx, uci_conf_dir);
@@ -222,11 +222,23 @@ configure_xtr(struct uci_context *ctx, struct uci_package *pck)
      * their names could appear in the rloc field of database mappings
      * or static map cache entries  */
     lcaf_ht = parse_lcafs(ctx,pck);
+    if (!lcaf_ht){
+        return (BAD);
+    }
 
     /* CREATE RLOCs sets HTABLE */
     no_addr_loct_list = glist_new_managed((glist_del_fct)no_addr_loct_del);
     rlocs_ht = parse_rlocs(ctx,pck,lcaf_ht,no_addr_loct_list);
+    if (!rlocs_ht){
+        glist_destroy(no_addr_loct_list);
+        return (BAD);
+    }
     rloc_set_ht = parse_rloc_sets(ctx,pck,rlocs_ht,lcaf_ht);
+    if (!rloc_set_ht){
+        glist_destroy(no_addr_loct_list);
+        shash_destroy(rloc_set_ht);
+        return (BAD);
+    }
 
     uci_foreach_element(&pck->sections, element) {
         sect = uci_to_section(element);
@@ -349,7 +361,7 @@ configure_xtr(struct uci_context *ctx, struct uci_package *pck)
 
             /* PROXY-ETR CONFIG */
 
-            if (strcmp(sect->type, "proxy-etr") == 0){
+            if (strcmp(sect->type, "proxy-etr-ipv4") == 0){
                 uci_address = (char *)uci_lookup_option_string(ctx, sect, "address");
                 if (uci_lookup_option_string(ctx, sect, "priority") != NULL){
                     uci_priority = strtol(uci_lookup_option_string(ctx, sect, "priority"),NULL,10);
@@ -366,7 +378,35 @@ configure_xtr(struct uci_context *ctx, struct uci_package *pck)
                     uci_weigth = 100;
                 }
 
-                if (add_proxy_etr_entry(xtr->petrs,uci_address,uci_priority,uci_weigth) != GOOD ){
+                if (add_proxy_etr_entry(xtr->petrs_ipv4,uci_address,uci_priority,uci_weigth) != GOOD ){
+                    OOR_LOG(LERR, "Can't add proxy-etr %s", uci_address);
+                }else{
+                    OOR_LOG(LDBG_1, "Added %s to proxy-etr list", uci_address);
+                }
+                continue;
+            }
+
+
+            /* PROXY-ETR CONFIG */
+
+            if (strcmp(sect->type, "proxy-etr-ipv6") == 0){
+                uci_address = (char *)uci_lookup_option_string(ctx, sect, "address");
+                if (uci_lookup_option_string(ctx, sect, "priority") != NULL){
+                    uci_priority = strtol(uci_lookup_option_string(ctx, sect, "priority"),NULL,10);
+                }else{
+                    OOR_LOG(LWRN,"Configuration file: No priority assigned to the proxy-etr \"%s\"."
+                            " Set default value: 10",uci_address);
+                    uci_priority = 10;
+                }
+                if (uci_lookup_option_string(ctx, sect, "weight") != NULL){
+                    uci_weigth = strtol(uci_lookup_option_string(ctx, sect, "weight"),NULL,10);
+                }else{
+                    OOR_LOG(LWRN,"Configuration file: No weight assigned to the proxy-etr \"%s\"."
+                            " Set default value: 100",uci_address);
+                    uci_weigth = 100;
+                }
+
+                if (add_proxy_etr_entry(xtr->petrs_ipv6,uci_address,uci_priority,uci_weigth) != GOOD ){
                     OOR_LOG(LERR, "Can't add proxy-etr %s", uci_address);
                 }else{
                     OOR_LOG(LDBG_1, "Added %s to proxy-etr list", uci_address);
@@ -394,7 +434,7 @@ configure_xtr(struct uci_context *ctx, struct uci_package *pck)
                 if (mapping == NULL){
                     OOR_LOG(LERR, "Can't add EID prefix %s. Discarded ...",
                             uci_lookup_option_string(ctx, sect, "eid_prefix"));
-                    continue;
+                    return (BAD);
                 }
                 map_loc_e = map_local_entry_new_init(mapping);
                 if (map_loc_e == NULL){
@@ -403,8 +443,7 @@ configure_xtr(struct uci_context *ctx, struct uci_package *pck)
                 }
 
                 if (xtr->fwd_policy->init_map_loc_policy_inf(
-                        xtr->fwd_policy_dev_parm,map_loc_e,NULL,
-                        xtr->fwd_policy->del_map_loc_policy_inf) != GOOD){
+                        xtr->fwd_policy_dev_parm,map_loc_e,NULL) != GOOD){
                     OOR_LOG(LERR, "Couldn't initiate forward information for mapping with EID: %s. Discarding it...",
                             lisp_addr_to_char(mapping_eid(mapping)));
                     map_local_entry_del(map_loc_e);
@@ -425,7 +464,7 @@ configure_xtr(struct uci_context *ctx, struct uci_package *pck)
                 if (mapping == NULL){
                     OOR_LOG(LERR, "Can't add static Map Cache entry with EID prefix %s. Discarded ...",
                             uci_lookup_option_string(ctx, sect, "eid_prefix"));
-                    continue;
+                    return (BAD);
                 }
                 if (mcache_lookup_exact(xtr->map_cache, mapping_eid(mapping)) == NULL){
                     if (tr_mcache_add_static_mapping(xtr, mapping) == GOOD){
@@ -446,10 +485,16 @@ configure_xtr(struct uci_context *ctx, struct uci_package *pck)
             }
     }
     /* Calculate forwarding info por proxy-etrs */
-    if (xtr->fwd_policy->init_map_cache_policy_inf(xtr->fwd_policy_dev_parm,xtr->petrs,
-            xtr->fwd_policy->del_map_cache_policy_inf) != GOOD){
-        OOR_LOG(LDBG_1, "Couldn't initiate routing info for PeTRs!.");
-        mcache_entry_del(xtr->petrs);
+    if (xtr->fwd_policy->init_map_cache_policy_inf(xtr->fwd_policy_dev_parm,xtr->petrs_ipv4) != GOOD){
+        OOR_LOG(LDBG_1, "Couldn't initiate routing info for PeTRs for IPv4 EIDs!.");
+        mcache_entry_del(xtr->petrs_ipv4);
+        return(BAD);
+    }
+
+    /* Calculate forwarding info por proxy-etrs */
+    if (xtr->fwd_policy->init_map_cache_policy_inf(xtr->fwd_policy_dev_parm,xtr->petrs_ipv6) != GOOD){
+        OOR_LOG(LDBG_1, "Couldn't initiate routing info for PeTRs for IPv6 EIDs!.");
+        mcache_entry_del(xtr->petrs_ipv6);
         return(BAD);
     }
 
@@ -504,11 +549,23 @@ configure_mn(struct uci_context *ctx, struct uci_package *pck)
      * their names could appear in the rloc field of database mappings
      * or static map cache entries  */
     lcaf_ht = parse_lcafs(ctx,pck);
+    if (!lcaf_ht){
+        return (BAD);
+    }
 
     /* CREATE RLOCs sets HTABLE */
     no_addr_loct_list = glist_new_managed((glist_del_fct)no_addr_loct_del);
     rlocs_ht = parse_rlocs(ctx,pck,lcaf_ht,no_addr_loct_list);
+    if (!rlocs_ht){
+        glist_destroy(no_addr_loct_list);
+        return (BAD);
+    }
     rloc_set_ht = parse_rloc_sets(ctx,pck,rlocs_ht,lcaf_ht);
+    if (!rloc_set_ht){
+        glist_destroy(no_addr_loct_list);
+        shash_destroy(rloc_set_ht);
+        return (BAD);
+    }
 
     uci_foreach_element(&pck->sections, element) {
         sect = uci_to_section(element);
@@ -631,7 +688,7 @@ configure_mn(struct uci_context *ctx, struct uci_package *pck)
 
         /* PROXY-ETR CONFIG */
 
-        if (strcmp(sect->type, "proxy-etr") == 0){
+        if (strcmp(sect->type, "proxy-etr-ipv4") == 0){
             uci_address = (char *)uci_lookup_option_string(ctx, sect, "address");
             if (uci_lookup_option_string(ctx, sect, "priority") != NULL){
                 uci_priority = strtol(uci_lookup_option_string(ctx, sect, "priority"),NULL,10);
@@ -648,7 +705,32 @@ configure_mn(struct uci_context *ctx, struct uci_package *pck)
                 uci_weigth = 100;
             }
 
-            if (add_proxy_etr_entry(xtr->petrs,uci_address,uci_priority,uci_weigth) != GOOD ){
+            if (add_proxy_etr_entry(xtr->petrs_ipv4,uci_address,uci_priority,uci_weigth) != GOOD ){
+                OOR_LOG(LERR, "Can't add proxy-etr %s", uci_address);
+            }else{
+                OOR_LOG(LDBG_1, "Added %s to proxy-etr list", uci_address);
+            }
+            continue;
+        }
+
+        if (strcmp(sect->type, "proxy-etr-ipv6") == 0){
+            uci_address = (char *)uci_lookup_option_string(ctx, sect, "address");
+            if (uci_lookup_option_string(ctx, sect, "priority") != NULL){
+                uci_priority = strtol(uci_lookup_option_string(ctx, sect, "priority"),NULL,10);
+            }else{
+                OOR_LOG(LWRN,"Configuration file: No priority assigned to the proxy-etr \"%s\"."
+                        " Set default value: 10",uci_address);
+                uci_priority = 10;
+            }
+            if (uci_lookup_option_string(ctx, sect, "weight") != NULL){
+                uci_weigth = strtol(uci_lookup_option_string(ctx, sect, "weight"),NULL,10);
+            }else{
+                OOR_LOG(LWRN,"Configuration file: No weight assigned to the proxy-etr \"%s\"."
+                        " Set default value: 100",uci_address);
+                uci_weigth = 100;
+            }
+
+            if (add_proxy_etr_entry(xtr->petrs_ipv6,uci_address,uci_priority,uci_weigth) != GOOD ){
                 OOR_LOG(LERR, "Can't add proxy-etr %s", uci_address);
             }else{
                 OOR_LOG(LDBG_1, "Added %s to proxy-etr list", uci_address);
@@ -676,7 +758,7 @@ configure_mn(struct uci_context *ctx, struct uci_package *pck)
             if (mapping == NULL){
                 OOR_LOG(LERR, "Can't add EID prefix %s. Discarded ...",
                         uci_lookup_option_string(ctx, sect, "eid_prefix"));
-                continue;
+                return (BAD);
             }
 
             map_loc_e = map_local_entry_new_init(mapping);
@@ -686,8 +768,7 @@ configure_mn(struct uci_context *ctx, struct uci_package *pck)
             }
 
             if ( xtr->fwd_policy->init_map_loc_policy_inf(
-                    xtr->fwd_policy_dev_parm,map_loc_e,NULL,
-                    xtr->fwd_policy->del_map_loc_policy_inf) != GOOD){
+                    xtr->fwd_policy_dev_parm,map_loc_e,NULL) != GOOD){
                 OOR_LOG(LERR, "Couldn't initiate forward information for mapping with EID: %s. Discarding it...",
                         lisp_addr_to_char(mapping_eid(mapping)));
                 map_local_entry_del(map_loc_e);
@@ -708,7 +789,7 @@ configure_mn(struct uci_context *ctx, struct uci_package *pck)
             if (mapping == NULL){
                 OOR_LOG(LERR, "Can't add static Map Cache entry with EID prefix %s. Discarded ...",
                         uci_lookup_option_string(ctx, sect, "eid_prefix"));
-                continue;
+                return (BAD);
             }
             if (mcache_lookup_exact(xtr->map_cache, mapping_eid(mapping)) == NULL){
                 if (tr_mcache_add_static_mapping(xtr, mapping) == GOOD){
@@ -730,10 +811,15 @@ configure_mn(struct uci_context *ctx, struct uci_package *pck)
     }
 
     /* Calculate forwarding info por proxy-etrs */
-    if (xtr->fwd_policy->init_map_cache_policy_inf(xtr->fwd_policy_dev_parm,xtr->petrs,
-            xtr->fwd_policy->del_map_cache_policy_inf) != GOOD){
-        OOR_LOG(LDBG_1, "Couldn't initiate routing info for PeTRs!.");
-        mcache_entry_del(xtr->petrs);
+    if (xtr->fwd_policy->init_map_cache_policy_inf(xtr->fwd_policy_dev_parm,xtr->petrs_ipv4) != GOOD){
+        OOR_LOG(LDBG_1, "Couldn't initiate routing info for PeTRs for IPv4 EIDs!.");
+        mcache_entry_del(xtr->petrs_ipv4);
+        return(BAD);
+    }
+
+    if (xtr->fwd_policy->init_map_cache_policy_inf(xtr->fwd_policy_dev_parm,xtr->petrs_ipv6) != GOOD){
+        OOR_LOG(LDBG_1, "Couldn't initiate routing info for PeTRs for IPv6 EIDs!.");
+        mcache_entry_del(xtr->petrs_ipv6);
         return(BAD);
     }
 
@@ -787,11 +873,23 @@ configure_rtr(struct uci_context *ctx, struct uci_package *pck)
      * their names could appear in the rloc field of database mappings
      * or static map cache entries  */
     lcaf_ht = parse_lcafs(ctx,pck);
+    if (!lcaf_ht){
+        return (BAD);
+    }
 
     /* CREATE RLOCs sets HTABLE */
     no_addr_loct_list = glist_new_managed((glist_del_fct)no_addr_loct_del);
     rlocs_ht = parse_rlocs(ctx,pck,lcaf_ht,no_addr_loct_list);
+    if (!rlocs_ht){
+        glist_destroy(no_addr_loct_list);
+        return (BAD);
+    }
     rloc_set_ht = parse_rloc_sets(ctx,pck,rlocs_ht,lcaf_ht);
+    if (!rloc_set_ht){
+        glist_destroy(no_addr_loct_list);
+        shash_destroy(rloc_set_ht);
+        return (BAD);
+    }
 
     uci_foreach_element(&pck->sections, element) {
         sect = uci_to_section(element);
@@ -897,7 +995,7 @@ configure_rtr(struct uci_context *ctx, struct uci_package *pck)
             if (mapping == NULL){
                 OOR_LOG(LERR, "Can't add static Map Cache entry with EID prefix %s. Discarded ...",
                         uci_lookup_option_string(ctx, sect, "eid_prefix"));
-                continue;
+                return (BAD);
             }
             if (mcache_lookup_exact(xtr->map_cache, mapping_eid(mapping)) == NULL){
                 if (tr_mcache_add_static_mapping(xtr, mapping) == GOOD){
@@ -995,11 +1093,23 @@ configure_ms(struct uci_context *ctx,struct uci_package *pck)
 
     /* create lcaf hash table */
     lcaf_ht = parse_lcafs(ctx,pck);
+    if (!lcaf_ht){
+        return (BAD);
+    }
 
     /* CREATE RLOCs sets HTABLE */
     no_addr_loct_list = glist_new_managed((glist_del_fct)no_addr_loct_del);
     rlocs_ht = parse_rlocs(ctx,pck,lcaf_ht,no_addr_loct_list);
+    if (!rlocs_ht){
+        glist_destroy(no_addr_loct_list);
+        return (BAD);
+    }
     rloc_set_ht = parse_rloc_sets(ctx,pck,rlocs_ht,lcaf_ht);
+    if (!rloc_set_ht){
+        glist_destroy(no_addr_loct_list);
+        shash_destroy(rloc_set_ht);
+        return (BAD);
+    }
 
     uci_foreach_element(&pck->sections, element) {
         sect = uci_to_section(element);
@@ -1016,17 +1126,8 @@ configure_ms(struct uci_context *ctx,struct uci_package *pck)
                 return(BAD);
             }
 
-            if (iface_address(iface, AF_INET) == NULL){
-                iface_setup_addr(iface, AF_INET);
-                data_plane->datap_add_iface_addr(iface,AF_INET);
-                lctrl->control_data_plane->control_dp_add_iface_addr(lctrl,iface,AF_INET);
-            }
-
-            if (iface_address(iface, AF_INET6) == NULL){
-                iface_setup_addr(iface, AF_INET6);
-                data_plane->datap_add_iface_addr(iface,AF_INET6);
-                lctrl->control_data_plane->control_dp_add_iface_addr(lctrl,iface,AF_INET6);
-            }
+            iface_configure (iface, AF_INET);
+            iface_configure (iface, AF_INET6);
         }
 
         /* LISP-SITE CONFIG */
@@ -1097,7 +1198,7 @@ configure_ms(struct uci_context *ctx,struct uci_package *pck)
             if (mapping == NULL){
                 OOR_LOG(LERR, "Can't create static register site for %s",
                         uci_lookup_option_string(ctx, sect, "eid_prefix"));
-                continue;
+                return (BAD);
             }
             if (mdb_lookup_entry_exact(ms->reg_sites_db, mapping_eid(mapping)) == NULL){
                 if (ms_add_registered_site_prefix(ms, mapping) == GOOD){
@@ -1142,7 +1243,7 @@ parse_mapping(struct uci_context *ctx, struct uci_section *sect,
     int uci_iid,iidmlen;
     glist_t *rloc_list;
     glist_entry_t *it;
-    lisp_xtr_t *xtr;
+    lisp_xtr_t *xtr = NULL;
 
     switch (dev->mode){
     case xTR_MODE:
@@ -1258,24 +1359,32 @@ parse_rlocs(struct uci_context *ctx, struct uci_package *pck, shash_t *lcaf_ht,
 
         if (strcmp(section->type, "rloc-address") == 0){
             uci_rloc_name = (char *)uci_lookup_option_string(ctx, section, "name");
+            if (uci_rloc_name == NULL){
+                OOR_LOG(LERR,"Configuration file: An rloc-address should have a name");
+                goto error;
+            }
             uci_address = (char *)uci_lookup_option_string(ctx, section, "address");
+            if (uci_address == NULL){
+                OOR_LOG(LERR,"Configuration file: No address assigned to the rloc \"%s\"",uci_rloc_name);
+                goto error;
+            }
             if (uci_lookup_option_string(ctx, section, "priority") == NULL){
                 OOR_LOG(LERR,"Configuration file: No priority assigned to the rloc \"%s\"",uci_rloc_name);
-                return (BAD);
+                goto error;
             }
             uci_priority = strtol(uci_lookup_option_string(ctx, section, "priority"),NULL,10);
             if (uci_lookup_option_string(ctx, section, "weight") == NULL){
                 OOR_LOG(LERR,"Configuration file: No weight assigned to the rloc \"%s\"",uci_rloc_name);
-                return (BAD);
+                goto error;
             }
             uci_weight = strtol(uci_lookup_option_string(ctx, section, "weight"),NULL,10);
 
             if (validate_priority_weight(uci_priority, uci_weight) != GOOD) {
-                continue;
+                goto error;
             }
             if (shash_lookup(rlocs_ht,uci_rloc_name) != NULL){
-                OOR_LOG(LDBG_1,"Configuration file: The RLOC %s is duplicated. Discarding ...", uci_rloc_name);
-                continue;
+                OOR_LOG(LDBG_1,"Configuration file: The RLOC %s is duplicated.", uci_rloc_name);
+                goto error;
             }
             addr_list = parse_lisp_addr(uci_address, lcaf_ht);
             if (addr_list == NULL || glist_size(addr_list) == 0){
@@ -1290,7 +1399,7 @@ parse_rlocs(struct uci_context *ctx, struct uci_package *pck, shash_t *lcaf_ht,
             if (lisp_addr_lafi(address) == LM_AFI_IPPREF){
                 OOR_LOG(LERR, "Configuration file: RLOC address can not be a prefix: %s ",
                         lisp_addr_to_char(address));
-                continue;
+                goto error;
             }
 
             /* Create a basic locator. Locaor or remote information will be added later according
@@ -1304,61 +1413,59 @@ parse_rlocs(struct uci_context *ctx, struct uci_package *pck, shash_t *lcaf_ht,
 
         if (strcmp(section->type, "rloc-iface") == 0){
             uci_rloc_name = (char *)uci_lookup_option_string(ctx, section, "name");
+            if (uci_rloc_name == NULL){
+                OOR_LOG(LERR,"Configuration file: An rloc-iface should have a name");
+                goto error;
+            }
             uci_iface_name = (char *)uci_lookup_option_string(ctx, section, "interface");
+            if (uci_iface_name == NULL){
+                OOR_LOG(LERR,"Configuration file: An rloc-iface should have an interface selected");
+                goto error;
+            }
             if (uci_lookup_option_string(ctx, section, "ip_version") == NULL){
                 OOR_LOG(LERR,"Configuration file: No afi assigned to the rloc \"%s\"",uci_rloc_name);
-                return (BAD);
+                goto error;
             }
             uci_afi = strtol(uci_lookup_option_string(ctx, section, "ip_version"),NULL,10);
             if (uci_lookup_option_string(ctx, section, "priority") == NULL){
                 OOR_LOG(LERR,"Configuration file: No priority assigned to the rloc \"%s\"",uci_rloc_name);
-                return (BAD);
+                goto error;
             }
             uci_priority = strtol(uci_lookup_option_string(ctx, section, "priority"),NULL,10);
             if (uci_lookup_option_string(ctx, section, "weight") == NULL){
                 OOR_LOG(LERR,"Configuration file: No weight assigned to the rloc \"%s\"",uci_rloc_name);
-                return (BAD);
+                goto error;
             }
             uci_weight = strtol(uci_lookup_option_string(ctx, section, "weight"),NULL,10);
 
             if (validate_priority_weight(uci_priority, uci_weight) != GOOD) {
-                continue;
+                goto error;
             }
 
             if (uci_afi != 4 && uci_afi !=6){
                 OOR_LOG(LERR, "Configuration file: The afi of the locator should be \"4\" (IPv4)"
                         " or \"6\" (IPv6)");
-                return (NULL);
+                goto error;
             }
 
             if (shash_lookup(rlocs_ht,uci_rloc_name) != NULL){
-                OOR_LOG(LDBG_1,"Configuration file: The RLOC %s is duplicated. Discarding ...", uci_rloc_name);
-                continue;
+                OOR_LOG(LDBG_1,"Configuration file: The RLOC %s is duplicated.", uci_rloc_name);
+                goto error;
             }
 
             /* Find the interface */
             if (!(iface = get_interface(uci_iface_name))) {
                 if (!(iface = add_interface(uci_iface_name))) {
-                    return (BAD);
+                    goto error;
                 }
             }
 
             if (uci_afi == 4){
-                if (iface_address(iface, AF_INET) == NULL){
-                    /* Configure address of the interface */
-                    iface_setup_addr(iface, AF_INET);
-                    data_plane->datap_add_iface_addr(iface,AF_INET);
-                    lctrl->control_data_plane->control_dp_add_iface_addr(lctrl,iface,AF_INET);
-                }
+                iface_configure (iface, AF_INET);
                 address = iface->ipv4_address;
                 afi = AF_INET;
             }else{
-                if (iface_address(iface, AF_INET6) == NULL){
-                    /* Configure address of the interface */
-                    iface_setup_addr(iface, AF_INET6);
-                    data_plane->datap_add_iface_addr(iface,AF_INET6);
-                    lctrl->control_data_plane->control_dp_add_iface_addr(lctrl,iface,AF_INET6);
-                }
+                iface_configure (iface, AF_INET6);
                 address = iface->ipv6_address;
                 afi = AF_INET6;
             }
@@ -1380,6 +1487,10 @@ parse_rlocs(struct uci_context *ctx, struct uci_package *pck, shash_t *lcaf_ht,
     }
 
     return (rlocs_ht);
+
+error:
+    shash_destroy(rlocs_ht);
+    return (NULL);
 }
 
 static shash_t *
@@ -1404,7 +1515,8 @@ parse_rloc_sets(struct uci_context *ctx, struct uci_package *pck, shash_t *rlocs
         if (strcmp(section->type, "rloc-set") == 0){
             uci_rloc_set_name = (char *)uci_lookup_option_string(ctx, section, "name");
             if (uci_rloc_set_name == NULL){
-                continue;
+                OOR_LOG(LERR,"Configuration file: An rloc-set should have a name");
+                goto error;
             }
             rloc_list = (glist_t*)shash_lookup(rloc_sets_ht,uci_rloc_set_name);
             if (rloc_list == NULL){
@@ -1416,9 +1528,9 @@ parse_rloc_sets(struct uci_context *ctx, struct uci_package *pck, shash_t *rlocs
                     continue;
                 }
             }else{
-                OOR_LOG(LWRN, "Configuration file: The RLOC set %s is duplicated. Discarding... ",
+                OOR_LOG(LERR, "Configuration file: The RLOC set %s is duplicated.",
                         uci_rloc_set_name);
-                continue;
+                goto error;
             }
             opt  = uci_lookup_option(ctx, section, "rloc_name");
             if (opt != NULL){
@@ -1427,7 +1539,7 @@ parse_rloc_sets(struct uci_context *ctx, struct uci_package *pck, shash_t *rlocs
                     if (loct == NULL){
                         OOR_LOG(LWRN,"Configuration file: The RLOC name %s of the RLOC set %s doesn't exist",
                                 element_loct->name, uci_rloc_set_name);
-                        continue;
+                        goto error;
                     }
 
                     if (glist_add_tail(loct,rloc_list)!=GOOD){
@@ -1437,12 +1549,17 @@ parse_rloc_sets(struct uci_context *ctx, struct uci_package *pck, shash_t *rlocs
             }else{
                 OOR_LOG(LWRN, "Configuration file: The RLOC set %s has no rlocs "
                         "associated.",uci_rloc_set_name);
+                goto error;
             }
 
         }
     }
 
     return (rloc_sets_ht);
+
+error:
+    shash_destroy(rloc_sets_ht);
+    return (NULL);
 }
 
 static shash_t *
@@ -1459,7 +1576,10 @@ parse_lcafs(struct uci_context *ctx, struct uci_package *pck)
         section = uci_to_section(element);
 
         if (strcmp(section->type, "elp-node") == 0){
-            parse_elp_node(ctx,section,lcaf_ht);
+            if (parse_elp_node(ctx,section,lcaf_ht)!=GOOD){
+                shash_destroy(lcaf_ht);
+                return (NULL);
+            }
         }
     }
 
@@ -1478,6 +1598,10 @@ parse_elp_node(struct uci_context *ctx, struct uci_section *section, shash_t *ht
     elp_node_t *elp_node;
 
     uci_elp_name = (char *)uci_lookup_option_string(ctx, section, "elp_name");
+    if (uci_elp_name == NULL){
+        OOR_LOG(LERR,"Configuration file: An ELP should have a name");
+        return (BAD);
+    }
     laddr = (lisp_addr_t *)shash_lookup(ht, uci_elp_name);
 
     if (laddr == NULL){
@@ -1500,10 +1624,14 @@ parse_elp_node(struct uci_context *ctx, struct uci_section *section, shash_t *ht
     elp_node->addr = lisp_addr_new();
 
     uci_address = (char *)uci_lookup_option_string(ctx, section, "address");
+    if (uci_address == NULL){
+        OOR_LOG(LERR,"Configuration file: An ELP node should have an address assigned");
+        return (BAD);
+    }
 
     if (lisp_addr_ip_from_char(uci_address, elp_node->addr) != GOOD) {
         elp_node_del(elp_node);
-        OOR_LOG(LDBG_1, "parse_elp_list: Couldn't parse ELP node %s",
+        OOR_LOG(LERR, "Configuration file: Couldn't parse ELP node %s",
                 uci_address);
         return (BAD);
     }
