@@ -20,6 +20,6 @@
 #ifndef OOR_CONFIG_UCI_H_
 #define OOR_CONFIG_UCI_H_
 
-int handle_config_file(char **uci_conf_file_path);
+int handle_config_file();
 
 #endif /* OOR_CONFIG_UCI_H_ */
