@@ -95,7 +95,7 @@ forward_mreq(lisp_ms_t *ms, lbuf_t *b, mapping_t *m)
     locator_t *loct = NULL;
     uconn_t fwd_uc;
 
-    ctrl = ctrl_dev_ctrl(&(ms->super));
+    ctrl = ctrl_dev_get_ctrl_t(&(ms->super));
 
     if ((ctrl_supported_afis(ctrl) & IPv4_SUPPORT) != 0){
     	loct = get_locator_with_afi(m, AF_INET);
@@ -435,10 +435,9 @@ ms_recv_map_register(lisp_ms_t *ms, lbuf_t *buf, uconn_t *uc)
                     OOR_LOG(LWRN, "Prefix %s has merge semantics",
                             lisp_addr_to_char(eid));
                 }
-                reg_pref->proxy_reply = MREG_PROXY_REPLY(hdr);
                 ms_dump_registered_sites(ms, LDBG_3);
             }
-
+            reg_pref->proxy_reply = MREG_PROXY_REPLY(hdr);
             /* update registration timer */
             lsite_entry_update_expiration_timer(ms, rsite);
         } else {
