@@ -22,6 +22,13 @@
 
 #include "../liblisp/lisp_address.h"
 
+int pref_is_addr_part_of_prefix(lisp_addr_t *addr, lisp_addr_t *pref);
+
+/*
+ * True if address belongs to the prefix
+ */
+int pref_is_addr_part_of_prefix(lisp_addr_t *addr, lisp_addr_t *pref);
+
 /*
  * If prefix b is contained in prefix a, then return TRUE. Otherwise return FALSE.
  * If both prefixs are the same it also returns TRUE
