@@ -29,6 +29,9 @@ char *get_char_from_xTR_ID (lisp_xtr_id *xtrid);
 /* Remove the address from the list not compatible with the local RLOCs */
 void addr_list_rm_not_compatible_addr(glist_t *addr_lst, int compatible_addr_flags);
 uint8_t is_compatible_addr(lisp_addr_t *addr, int compatible_addr_flags);
+void str_rm_spaces(char *src, char *dst);
+void str_rm_double_spaces(char *src, char *dst);
+
 
 #endif /* UTIL_H_ */
 
